@@ -4,6 +4,7 @@
  *      Copyright 2009 Juergen Hötzel <juergen@archlinux.org>
  *                2015 Henry Gebhardt <hsggebhardt@googlemail.com>
  *                2015 Stanislav Kozina, Ersin <xersin@users.sf.net>
+ *                2016-2017 Andriy Grytsenko <andrej@rep.kiev.ua>
  *
  * 	Parts shameless stolen and glibified from acpi package
  * 	Copyright (C) 2001  Grahame Bowland <grahame@angrygoats.net>
@@ -166,6 +167,7 @@ static gboolean battery_inserted(gchar* path)
 battery* battery_update(battery *b)
 {
     gchar *gctmp;
+    int promille;
 
     if (b == NULL)
         return NULL;
@@ -252,23 +254,29 @@ battery* battery_update(battery *b)
     }
 #endif
 
-    if (b->charge_full < MIN_CAPACITY)
-        b->percentage = 0;
-    else {
-        int promille = (b->charge_now * 1000) / b->charge_full;
-        b->percentage = (promille + 5) / 10; /* round properly */
-    }
+    if (b->charge_now != -1 && b->charge_full != -1)
+        promille = (b->charge_now * 1000) / b->charge_full;
+    else if (b->energy_full != -1 && b->energy_now != -1)
+        /* no charge data, let try energy instead */
+        promille = (b->energy_now * 1000) / b->energy_full;
+    else
+        promille = 0;
+
+    b->percentage = (promille + 5) / 10; /* round properly */
     if (b->percentage > 100)
         b->percentage = 100;
 
-
-    if (b->current_now == -1) {
+    if (b->power_now < -1)
+        b->power_now = - b->power_now;
+    if (b->current_now == -1 && b->power_now == -1) {
         //b->poststr = "rate information unavailable";
         b->seconds = -1;
     } else if (!strcasecmp(b->state, "charging")) {
         if (b->current_now > MIN_PRESENT_RATE) {
             b->seconds = 3600 * (b->charge_full - b->charge_now) / b->current_now;
             //b->poststr = " until charged";
+        } else if (b->power_now > 0) {
+            b->seconds = 3600 * (b->energy_full - b->energy_now) / b->power_now;
         } else {
             //b->poststr = "charging at zero rate - will never fully charge.";
             b->seconds = -1;
@@ -277,6 +285,8 @@ battery* battery_update(battery *b)
         if (b->current_now > MIN_PRESENT_RATE) {
             b->seconds = 3600 * b->charge_now / b->current_now;
             //b->poststr = " remaining";
+        } else if (b->power_now > 0) {
+            b->seconds = 3600 * b->energy_now / b->power_now;
         } else {
             //b->poststr = "discharging at zero rate - will never fully discharge.";
             b->seconds = -1;
@@ -295,15 +305,8 @@ battery *battery_get(int battery_number) {
     const gchar *entry;
     gchar *batt_name = NULL;
     gchar *batt_path = NULL;
-    GDir * dir = g_dir_open( ACPI_PATH_SYS_POWER_SUPPLY, 0, &error );
+    GDir * dir;
     battery *b = NULL;
-    int i;
-
-    if ( dir == NULL )
-    {
-        g_warning( "NO ACPI/sysfs support in kernel: %s", error->message );
-        return NULL;
-    }
 
     /* Try the expected path in sysfs first */
     batt_name = g_strdup_printf(ACPI_BATTERY_DEVICE_NAME "%d", battery_number);
@@ -324,13 +327,20 @@ battery *battery_get(int battery_number) {
     g_free(batt_path);
 
     if (b != NULL)
-        goto done;
+        return b;
 
     /*
      * We didn't find the expected path in sysfs.
-     * Walk the dir and blindly return n-th entry.
+     * Walk the dir and return any battery.
      */
-    i = 0;
+    dir = g_dir_open( ACPI_PATH_SYS_POWER_SUPPLY, 0, &error );
+    if ( dir == NULL )
+    {
+        g_warning( "NO ACPI/sysfs support in kernel: %s", error->message );
+        g_error_free(error);
+        return NULL;
+    }
+
     while ( ( entry = g_dir_read_name (dir) ) != NULL )
     {
         b = battery_new();
@@ -339,9 +349,7 @@ battery *battery_get(int battery_number) {
 
         /* We're looking for a battery with the selected ID */
         if (b->type_battery == TRUE) {
-            if (i == battery_number)
-                break;
-            i++;
+            break;
         }
         battery_free(b);
         b = NULL;
@@ -349,9 +357,10 @@ battery *battery_get(int battery_number) {
     if (b != NULL)
         g_warning( "Battery entry " ACPI_BATTERY_DEVICE_NAME "%d not found, using %s",
             battery_number, b->path);
+        // FIXME: update config?
     else
         g_warning( "Battery %d not found", battery_number );
-done:
+
     g_dir_close( dir );
     return b;
 }
